package Alzabo::Runtime::Column;

use strict;
use vars qw($VERSION);

use Alzabo::Runtime;

use base qw(Alzabo::Column);

$VERSION = sprintf '%2d.%02d', q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/;

1;

__END__

=head1 NAME

Alzabo::Runtime::Column - Column objects

=head1 SYNOPSIS

  use Alzabo::Runtime::Column;

=head1 DESCRIPTION

Right now this class is empty and is just a placeholder.

=head1 AUTHOR

Dave Rolsky, <autarch@urth.org>

=cut
