# This is just to test whether this stuff compiles.

use Alzabo::Config;

use Alzabo::ChangeTracker;

use Alzabo::ObjectCacheIPC;

use Alzabo;

use Alzabo::Create;

use Alzabo::Runtime;

use Alzabo::Driver;
use Alzabo::Driver::MySQL;
use Alzabo::Driver::Oracle;
use Alzabo::Driver::PostgreSQL;

use Alzabo::RDBMSRules;
use Alzabo::RDBMSRules::MySQL;
use Alzabo::RDBMSRules::Oracle;
use Alzabo::RDBMSRules::PostgreSQL;



print "1..1\n";
print "ok 1\n";