package Alzabo::RDBMSRules::Oracle;

use strict;
use vars qw($VERSION);

$VERSION = '0.01';


# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Alzabo::RDBMSRules::Oracle - Perl extension for blah blah blah

=head1 SYNOPSIS

  use Alzabo::RDBMSRules::Oracle;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for Alzabo::RDBMSRules::Oracle was created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head1 AUTHOR

A. U. Thor, a.u.thor@a.galaxy.far.far.away

=head1 SEE ALSO

perl(1).

=cut
